import { Injectable } from '@angular/core';
import { Observable } from 'rxjs/Observable';
import 'rxjs/Rx';
import { TransportContainerService } from '../transport/transport-container.service';
import { CommandUtilityService } from '../utilities/command-utility.service';
export var FileService = (function () {
    function FileService(transport) {
        this.transport = transport;
        this.commandUtilService = new CommandUtilityService();
    }
    //Set the output voltage of the specified DC power supply channel.
    FileService.prototype.write = function (location, path, file, filePosition) {
        var _this = this;
        filePosition = filePosition == undefined ? 0 : filePosition;
        var command = {
            file: [{
                    command: 'write',
                    type: location,
                    path: path,
                    filePosition: filePosition,
                    binaryOffset: 0,
                    binaryLength: file.byteLength
                }]
        };
        return Observable.create(function (observer) {
            var dataToSend = _this.commandUtilService.createChunkedArrayBuffer(command, file);
            _this.transport.writeRead('/', dataToSend, 'binary').subscribe(function (arrayBuffer) {
                var data;
                try {
                    var stringify = String.fromCharCode.apply(null, new Uint8Array(arrayBuffer.slice(0)));
                    data = JSON.parse(stringify);
                }
                catch (e) {
                    observer.error(e);
                    return;
                }
                if (data == undefined || data.agent != undefined) {
                    observer.error(data);
                    return;
                }
                if (data.file == undefined || data.file[0] == undefined || data.file[0].statusCode !== 0) {
                    observer.error(data);
                    return;
                }
                observer.next(data);
                observer.complete();
            }, function (err) {
                observer.error(err);
            }, function () {
                observer.complete();
            });
        });
    };
    FileService.prototype.listDir = function (location, path) {
        var command = {
            file: [{
                    command: 'listdir',
                    type: location,
                    path: path
                }]
        };
        return this.genericResponse(command);
    };
    FileService.prototype.getFileSize = function (location, path) {
        var command = {
            file: [{
                    command: 'getFileSize',
                    type: location,
                    path: path
                }]
        };
        return this.genericResponse(command);
    };
    FileService.prototype.getCurrentState = function () {
        var command = {
            file: [{
                    command: 'getCurrentState'
                }]
        };
        return this.genericResponse(command);
    };
    FileService.prototype.delete = function (location, paths) {
        var command = {
            file: []
        };
        var p = (typeof paths === 'string') ? [paths] : paths;
        p.forEach(function (path) {
            command.file.push({
                command: 'delete',
                type: location,
                path: path
            });
        });
        return this.genericResponse(command);
    };
    //Set the output voltage of the specified DC power supply channel.
    FileService.prototype.read = function (location, path, filePosition, length, timeoutOverride) {
        var _this = this;
        var command = {
            file: [{
                    command: 'read',
                    type: location,
                    path: path,
                    filePosition: filePosition,
                    requestedLength: length
                }]
        };
        return Observable.create(function (observer) {
            _this.transport.writeRead('/', JSON.stringify(command), 'json', timeoutOverride).subscribe(function (arrayBuffer) {
                _this.commandUtilService.observableParseChunkedTransfer(arrayBuffer, 'u8').subscribe(function (data) {
                    var jsonObject = data.json;
                    var binaryData = data.typedArray;
                    if (data == undefined || data.agent != undefined) {
                        observer.error(data);
                        return;
                    }
                    if (jsonObject.file == undefined || jsonObject.file[0] == undefined || jsonObject.file[0].statusCode !== 0) {
                        observer.error(data);
                        return;
                    }
                    observer.next({
                        jsonObject: jsonObject,
                        file: _this.safeStringBinary(binaryData)
                    });
                    observer.complete();
                }, function (err) {
                    observer.error(err);
                }, function () { });
            }, function (err) {
                observer.error(err);
            }, function () {
                observer.complete();
            });
        });
    };
    FileService.prototype.safeStringBinary = function (binaryData) {
        var returnString = '';
        for (var i = 0; i < binaryData.length; i++) {
            returnString += String.fromCharCode(binaryData[i]);
        }
        return returnString;
    };
    FileService.prototype.genericResponse = function (command) {
        var _this = this;
        return Observable.create(function (observer) {
            _this.transport.writeRead('/', JSON.stringify(command), 'json').subscribe(function (arrayBuffer) {
                var data;
                try {
                    var stringify = String.fromCharCode.apply(null, new Uint8Array(arrayBuffer.slice(0)));
                    data = JSON.parse(stringify);
                }
                catch (e) {
                    observer.error(e);
                    return;
                }
                if (data == undefined || data.agent != undefined) {
                    observer.error(data);
                    return;
                }
                if (data.file == undefined || (data.file[0] !== undefined && data.file[0].statusCode !== 0) || (data.file[0] === undefined && data.file.statusCode !== 0)) {
                    observer.error(data);
                    return;
                }
                observer.next(data);
                observer.complete();
            }, function (err) {
                observer.error(err);
            }, function () {
                observer.complete();
            });
        });
    };
    FileService.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    FileService.ctorParameters = [
        { type: TransportContainerService, },
    ];
    return FileService;
}());
//# sourceMappingURL=file.service.js.map