import { Injectable } from '@angular/core';
import { Observable } from 'rxjs/Observable';
import 'rxjs/Rx';
import { GenericInstrumentService } from '../../generic-instrument.service';
import { CommandUtilityService } from '../../../../utilities/command-utility.service';
export var LoggerCommandService = (function () {
    function LoggerCommandService(_loggerInstrumentRef) {
        this.instrumentRef = _loggerInstrumentRef;
        this.commandUtilityService = new CommandUtilityService();
    }
    LoggerCommandService.prototype.setParametersJson = function (chans, maxSampleCount, sampleFreq, startDelay, storageLocation, logOnBoot, service, apiKey, uri, averages) {
        var command = {
            "log": {
                "daq": {
                    "command": "setParameters",
                    "maxSampleCount": maxSampleCount,
                    "startDelay": Math.round(startDelay * Math.pow(10, 12)),
                    "sampleFreq": Math.round(sampleFreq * 1000000),
                    "storageLocation": storageLocation,
                    "logOnBoot": logOnBoot,
                    "service": service,
                    "key": apiKey,
                    "uri": uri,
                    "channels": []
                }
            }
        };
        chans.forEach(function (element, index, array) {
            var channelSettings = {};
            channelSettings[chans[index]] = {
                average: averages[index]
            };
            command.log.daq.channels.push(channelSettings);
        });
        return command;
    };
    LoggerCommandService.prototype.runParse = function (chan, responseObject) {
        return 'Channel ' + chan + ' ' + responseObject.command + ' successful';
    };
    LoggerCommandService.prototype.stopParse = function (chan, responseObject) {
        return 'Channel ' + chan + ' ' + responseObject.command + ' successful';
    };
    LoggerCommandService.prototype.runJson = function () {
        var command = {
            "log": {
                "daq": {
                    "command": "run"
                }
            }
        };
        return command;
    };
    LoggerCommandService.prototype.stopJson = function () {
        var command = {
            "log": {
                "daq": {
                    "command": "stop"
                }
            }
        };
        return command;
    };
    LoggerCommandService.prototype.readJson = function (chans, startIndex, count) {
        if (count === void 0) { count = -1; }
        var command = {
            "log": {
                "daq": {
                    "command": "read",
                    "channels": chans,
                    "startIndex": startIndex,
                    "count": count
                }
            }
        };
        return command;
    };
    LoggerCommandService.prototype.getCurrentStateJson = function () {
        var command = {
            "log": {
                "daq": {
                    "command": "getCurrentState"
                }
            }
        };
        return command;
    };
    LoggerCommandService.prototype.setParameters = function (chans, maxSampleCount, sampleFreq, startDelay, storageLocation, logOnBoot, service, apiKey, uri, averages) {
        var command = this.setParametersJson(chans, maxSampleCount, sampleFreq, startDelay, storageLocation, logOnBoot, service, apiKey, uri, averages);
        return this.instrumentRef._genericResponseHandler(command);
    };
    LoggerCommandService.prototype.run = function () {
        var command = this.runJson();
        return this.instrumentRef._genericResponseHandler(command);
    };
    LoggerCommandService.prototype.stop = function () {
        var command = this.stopJson();
        return this.instrumentRef._genericResponseHandler(command);
    };
    LoggerCommandService.prototype.read = function (chans, startIndex, count) {
        var _this = this;
        var command = this.readJson(chans, startIndex, count);
        return Observable.create(function (observer) {
            _this.instrumentRef.transport.writeRead('/', JSON.stringify(command), 'binary')
                .flatMap(function (data) {
                return _this.commandUtilityService.observableParseChunkedTransfer(data);
            })
                .retryWhen(function (errors) { return errors.map(function (err) {
                if (err.payload.log.daq.statusCode === 2684354570) {
                    return {}; // retry request
                }
                else
                    throw err; // properly error
            }); })
                .subscribe(function (data) {
                var returnObject = {
                    cmdRespObj: data.json,
                    instruments: {}
                };
                var command = data.json;
                var typedArray = data.typedArray;
                console.log(command);
                for (var instrument in command.log) {
                    returnObject.instruments[instrument] = {};
                    for (var _i = 0, _a = command.log.daq.channels; _i < _a.length; _i++) {
                        var channel = _a[_i];
                        returnObject.instruments[instrument][channel] = {};
                        Object.assign(returnObject.instruments[instrument][channel], command.log[instrument]);
                        returnObject.instruments[instrument][channel]['data'] = [];
                    }
                    // iterate over the data in the typed array, and pack each index into the respective channel buffer
                    for (var dataIndex = 0; dataIndex < typedArray.length; dataIndex += command.log.daq.channels.length) {
                        for (var chanIndex = 0; chanIndex < command.log.daq.channels.length; chanIndex++) {
                            var channel = command.log.daq.channels[chanIndex];
                            var scaledValue = typedArray[dataIndex + chanIndex] / 1000;
                            returnObject.instruments[instrument][channel]['data'].push(scaledValue);
                        }
                    }
                }
                observer.next(returnObject);
                observer.complete();
            }, function (err) {
                console.log(err);
                observer.error(err);
            }, function () { });
        });
    };
    LoggerCommandService.prototype.getCurrentState = function () {
        var command = this.getCurrentStateJson();
        return this.instrumentRef._genericResponseHandler(command);
    };
    LoggerCommandService.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    LoggerCommandService.ctorParameters = [
        { type: GenericInstrumentService, },
    ];
    return LoggerCommandService;
}());
//# sourceMappingURL=logger-command.service.js.map