import { Injectable } from '@angular/core';
import { Observable } from 'rxjs/Observable';
import 'rxjs/Rx';
import { GenericInstrumentService } from '../generic-instrument.service';
import { CommandUtilityService } from '../../../utilities/command-utility.service';
export var LoggerCommandService = (function () {
    function LoggerCommandService(_loggerInstrumentRef) {
        this.instrumentRef = _loggerInstrumentRef;
        this.commandUtilityService = new CommandUtilityService();
    }
    LoggerCommandService.prototype.analogSetParametersJson = function (chans, maxSampleCounts, gains, vOffsets, sampleFreqs, startDelays, overflows, storageLocations, uris) {
        var command = {
            "log": {
                "analog": {}
            }
        };
        chans.forEach(function (element, index, array) {
            command.log.analog[chans[index]] =
                [
                    {
                        "command": "setParameters",
                        "maxSampleCount": maxSampleCounts[index],
                        "gain": gains[index],
                        "vOffset": Math.round(vOffsets[index] * 1000),
                        "sampleFreq": Math.round(sampleFreqs[index] * 1000000),
                        "startDelay": Math.round(startDelays[index] * Math.pow(10, 12)),
                        "overflow": overflows[index],
                        "storageLocation": storageLocations[index],
                        "uri": uris[index]
                    }
                ];
        });
        return command;
    };
    LoggerCommandService.prototype.daqSetParametersJson = function (chans, maxSampleCount, sampleFreq, startDelay, storageLocation, uri, averages) {
        var command = {
            "log": {
                "daq": {
                    "command": "setParameters",
                    "maxSampleCount": maxSampleCount,
                    "startDelay": startDelay,
                    "sampleFreq": sampleFreq,
                    "storageLocation": storageLocation,
                    "uri": uri,
                    "channels": []
                }
            }
        };
        chans.forEach(function (element, index, array) {
            var channelSettings = {};
            channelSettings[chans[index]] = {
                average: averages[index]
            };
            command.log.daq.channels.push(channelSettings);
        });
        return command;
    };
    LoggerCommandService.prototype.analogSetParametersParse = function (chan, responseObject) {
        return 'Channel ' + chan + ' ' + responseObject.command + ' successful';
    };
    LoggerCommandService.prototype.digitalSetParametersParse = function (chan, responseObject) {
        return 'Channel ' + chan + ' ' + responseObject.command + ' successful';
    };
    LoggerCommandService.prototype.runParse = function (chan, responseObject) {
        return 'Channel ' + chan + ' ' + responseObject.command + ' successful';
    };
    LoggerCommandService.prototype.stopParse = function (chan, responseObject) {
        return 'Channel ' + chan + ' ' + responseObject.command + ' successful';
    };
    LoggerCommandService.prototype.digitalSetParametersJson = function (chans, maxSampleCounts, sampleFreqs, startDelays, overflows, storageLocations, uris, bitMasks) {
        var command = {
            "log": {
                "digital": {}
            }
        };
        chans.forEach(function (element, index, array) {
            command.log.digital[chans[index]] =
                [
                    {
                        "command": "setParameters",
                        "bitMask": bitMasks[index],
                        "maxSampleCount": maxSampleCounts[index],
                        "sampleFreq": Math.round(sampleFreqs[index] * 1000000),
                        "startDelay": Math.round(startDelays[index] * Math.pow(10, 12)),
                        "overflow": overflows[index],
                        "storageLocation": storageLocations[index],
                        "uri": uris[index]
                    }
                ];
        });
        return command;
    };
    LoggerCommandService.prototype.runJson = function (instrument, chans) {
        var command = {
            "log": {}
        };
        command.log[instrument] = {};
        chans.forEach(function (element, index, array) {
            command.log[instrument][chans[index]] =
                [
                    {
                        command: "run"
                    }
                ];
        });
        return command;
    };
    LoggerCommandService.prototype.stopJson = function (instrument, chans) {
        var command = {
            "log": {}
        };
        command.log[instrument] = {};
        chans.forEach(function (element, index, array) {
            command.log[instrument][chans[index]] =
                [
                    {
                        command: "stop"
                    }
                ];
        });
        return command;
    };
    LoggerCommandService.prototype.readJson = function (instrument, chans, startIndices, counts) {
        var command = {
            "log": {}
        };
        command.log[instrument] = {};
        chans.forEach(function (element, index, array) {
            command.log[instrument][chans[index]] =
                [
                    {
                        command: "read",
                        startIndex: startIndices[index],
                        count: counts[index]
                    }
                ];
        });
        return command;
    };
    LoggerCommandService.prototype.getCurrentStateJson = function (instrument, chans) {
        var command = {
            "log": {}
        };
        if (instrument === 'daq') {
            command.log[instrument] = { command: 'getCurrentState' };
        }
        else {
            command.log[instrument] = {};
            chans.forEach(function (element, index, array) {
                command.log[instrument][chans[index]] =
                    [
                        {
                            command: "getCurrentState"
                        }
                    ];
            });
        }
        return command;
    };
    LoggerCommandService.prototype.analogSetParameters = function (chans, maxSampleCounts, gains, vOffsets, sampleFreqs, startDelays, overflows, storageLocations, uris) {
        var command = this.analogSetParametersJson(chans, maxSampleCounts, gains, vOffsets, sampleFreqs, startDelays, overflows, storageLocations, uris);
        return this.instrumentRef._genericResponseHandler(command);
    };
    LoggerCommandService.prototype.daqSetParameters = function (chans, maxSampleCount, sampleFreq, startDelay, storageLocation, uri, averages) {
        var command = this.daqSetParametersJson(chans, maxSampleCount, sampleFreq, startDelay, storageLocation, uri, averages);
        return this.instrumentRef._genericResponseHandler(command);
    };
    LoggerCommandService.prototype.digitalSetParameters = function (chans, maxSampleCounts, sampleFreqs, startDelays, overflows, storageLocations, uris, bitMasks) {
        var command = this.digitalSetParametersJson(chans, maxSampleCounts, sampleFreqs, startDelays, overflows, storageLocations, uris, bitMasks);
        return this.instrumentRef._genericResponseHandler(command);
    };
    LoggerCommandService.prototype.run = function (instrument, chans) {
        var command = this.runJson(instrument, chans);
        return this.instrumentRef._genericResponseHandler(command);
    };
    LoggerCommandService.prototype.stop = function (instrument, chans) {
        var command = this.stopJson(instrument, chans);
        return this.instrumentRef._genericResponseHandler(command);
    };
    LoggerCommandService.prototype.read = function (instrument, chans, startIndices, counts) {
        var _this = this;
        var command = this.readJson(instrument, chans, startIndices, counts);
        return Observable.create(function (observer) {
            _this.instrumentRef.transport.writeRead('/', JSON.stringify(command), 'binary')
                .flatMap(function (data) {
                return _this.commandUtilityService.observableParseChunkedTransfer(data);
            })
                .subscribe(function (data) {
                var returnObject = {
                    cmdRespObj: data.json,
                    instruments: {}
                };
                var command = data.json;
                console.log(command);
                for (var instrument_1 in command.log) {
                    returnObject.instruments[instrument_1] = {};
                    for (var channel in command.log[instrument_1]) {
                        returnObject.instruments[instrument_1][channel] = {};
                        if (command.log[instrument_1][channel][0].statusCode > 0) {
                            observer.error('StatusCode error: ' + instrument_1 + ' Ch ' + channel);
                            return;
                        }
                        if (command.log[instrument_1][channel][0].binaryLength === 0) {
                            observer.error('No data received on ' + instrument_1 + ' Ch ' + channel);
                            return;
                        }
                        var binaryOffset = command.log[instrument_1][channel][0].binaryOffset / 2;
                        var binaryData = data.typedArray.slice(binaryOffset, binaryOffset + command.log[instrument_1][channel][0].binaryLength / 2);
                        var untypedArray = Array.prototype.slice.call(binaryData);
                        var scaledArray = untypedArray.map(function (voltage) {
                            return voltage / 1000;
                        });
                        Object.assign(returnObject.instruments[instrument_1][channel], command.log[instrument_1][channel][0]);
                        returnObject.instruments[instrument_1][channel]['data'] = scaledArray;
                    }
                }
                observer.next(returnObject);
                observer.complete();
            }, function (err) {
                console.log(err);
                observer.error(err);
            }, function () { });
        });
    };
    LoggerCommandService.prototype.getCurrentState = function (instrument, chans) {
        var command = this.getCurrentStateJson(instrument, chans);
        return this.instrumentRef._genericResponseHandler(command);
    };
    LoggerCommandService.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    LoggerCommandService.ctorParameters = [
        { type: GenericInstrumentService, },
    ];
    return LoggerCommandService;
}());
//# sourceMappingURL=logger-command.service.js.map